import datetime
from pandas import read_csv
from pandas._testing import assert_frame_equal
from pfit.CSVColMelter import CSVMelter
import unittest

class TestCSVMelter(unittest.TestCase):
    def setUp(self):
        self.csvMelter = CSVMelter()
        self.date = "2020-05-21"
        self.time = "08:37:55"
        self.isoTime = "2020-05-21T08:37:55-07:00"
        self.dateObj = datetime.datetime(year=2020, month=5, day=21, hour=8, minute=37, second=55, tzinfo=self.csvMelter.areaTimezone)
        self.timeObj = datetime.time(hour=8, minute=37, second=55)
        
    def test_get_iso_format(self):
        self.assertEqual(self.csvMelter.getISOFormat(self.date, self.time), self.isoTime)
        self.assertEqual(self.csvMelter.getISOFormat(self.dateObj, self.timeObj), self.isoTime)
        
        with self.assertRaises(ValueError):
            self.csvMelter.getISOFormat("2010/07/30", self.time)
            self.csvMelter.getISOFormat("2010/07/35", self.time)
            self.csvMelter.getISOFormat(self.date, "183459")
            self.csvMelter.getISOFormat(self.date, "83:78:00")
            self.csvMelter.getISOFormat(self.date, "0:0:99")  

    def test_parse_time_format(self):
        self.assertEqual(self.csvMelter.parseTimeFormat(datetime.datetime.combine(self.dateObj, self.timeObj)), (self.date, self.time))
        
        with self.assertRaises(ValueError):
            self.csvMelter.parseTimeFormat(self.date)
            self.csvMelter.parseTimeFormat(self.time)
            self.csvMelter.parseTimeFormat("2020-35-21T08:37:55-07:00")
            self.csvMelter.parseTimeFormat("2020-35-21 08:37:55")
            self.csvMelter.parseTimeFormat("2020-35-21T08:37:55-07:00")
            self.csvMelter.parseTimeFormat("2020-05-21T08:37:65-07:00")
            self.csvMelter.parseTimeFormat("2020-05-21 08:37:65-07:00")
            self.csvMelter.parseTimeFormat("2020-05-21 08:37:15-07:00")
            self.csvMelter.parseTimeFormat("2020-05-21 08:37:15")
            
    def test_melt_dataframe(self):
        wideFile = read_csv("test/sample/NTGS_Wide.csv", keep_default_na=False, float_precision="round_trip", parse_dates={"time": [4,5]}, date_parser=self.csvMelter.getISOFormat)
        wideMelt = read_csv("test/sample/NTGS_Wide_Melted.csv", keep_default_na=False, float_precision="round_trip")
        resMelt = self.csvMelter.meltDataframe(wideFile)
        assert_frame_equal(resMelt, wideMelt, check_exact=True)
    
    def test_stringify_depths(self):
        depthDict = {5: "5_m",
                    5.1: "5.1_m",
                    0.05: "0.05_m",
                    0: "0_m",
                    1.005: "1.005_m",
                    2.0: "2_m",
                    -5: "-5_m"}
        depths = ["5_m", "5.1_m", "0.05_m", "0_m", "1.005_m", "2_m", "-5_m"]
        resDict, resDepths = self.csvMelter.stringifyDepths(["colA", "colB", 5, 5.1, 0.05, 0, 1.005, "colH", 2.0, -5.0])
        self.assertDictEqual(depthDict, resDict)
        self.assertListEqual(depths, resDepths)
        
    def test_long_to_wide(self):
        longFile = read_csv("test/sample/NTGS_Long.csv", keep_default_na=False, float_precision="round_trip")
        longWidened = read_csv("test/sample/NTGS_Long_Widened.csv", keep_default_na=False, float_precision="round_trip")
        resWide = self.csvMelter.longToWide(longFile)
        assert_frame_equal(resWide, longWidened, check_exact=True)

if __name__ == "__main__":
    unittest.main()



from pfit.NTGSTemperatureMetadataHandler import NTGSTemperatureMetadataHandler
from pfit.TabularMetadataHandler import TabularMetadataHandler
from pfit.NTGS_to_NetCDF import NtgsThermalDataset
from pfit.CSVColMelter import CSVMelter

# ncfile = r"C:\Users\Nick\Downloads\201709\TST_HE_2_DC.nc"
# melted_csv = r"C:\Users\Nick\Downloads\201709\melted_TST HE 2 DC.csv"

# nc_from_melted(ncfile, melted_csv, None, metadata_id=None)

# M = NTGSTemperatureMetadataHandler(r"C:\Users\Nick\Carleton University\NSERC PermafrostNet - Operations - Operations\900_Data\904_DataSciencePlatform\90405_ntgsData\NTGSrefno_2017-009_2020August18\2017-009_2017-009\2017-009_Metadata\NWT_Open_Report_2017-009_Metadata.xlsx")

# M.write_attributes(ncfile, "TST HE2 DC")
# M.write_variables(ncfile, "TST HE2 DC")


##

metadata = r"C:/Users/Nick/Downloads/ESJ_Metadata.xlsx"
data_file = r"C:\Users\Nick\Desktop\clean\pink_west_merged_clean1.csv"
D = NtgsThermalDataset(data_file=data_file, metadata=None, timezoneOffset=-8)


q = D.to_netcdf("C:/Users/Nick/ESJ2.nc")
T = TabularMetadataHandler.from_xlsx(metadata, 0, "index")
T.write_attributes(q, "", loc_lookup="site_name")


# from pathlib import Path
# folder = Path(r"C:\Users\Nick\Desktop\clean")
# for file in folder.glob("*.csv"):
#     print(file)
#     D = NtgsThermalDataset(data_file=data_file, metadata=None, timezoneOffset=-8)


import pandas as pd
from pfit.FileDataChecker import FileDataChecker
import unittest

class TestFileDataChecker(unittest.TestCase):
    def setUp(self):
        self.checker = FileDataChecker()
        self.rx_time = self.checker.TIME_RGX
        self.rx_metre = self.checker.METRE_RGX
        self.expected = ["project_name", "site_id", "latitude", "longitude", "date_YYYY-MM-DD", "time_HH:MM:SS"]
        
    def test_time_regex(self):
        self.assertRegex("00:00:00", self.rx_time)
        self.assertRegex("0:00:00", self.rx_time)
        self.assertRegex("12:59:59", self.rx_time)
        self.assertRegex("12:59:00", self.rx_time)
        self.assertRegex("13:00:00", self.rx_time)
        self.assertRegex("23:00:00", self.rx_time)
        self.assertRegex("23:59:59", self.rx_time)
        self.assertNotRegex("24:00:00", self.rx_time)
        self.assertNotRegex("99:99:99", self.rx_time)
        self.assertNotRegex("01:00:60", self.rx_time)
        self.assertNotRegex("10:00:99", self.rx_time)
        self.assertNotRegex("10:95:58", self.rx_time)
        self.assertNotRegex("-12:22:58", self.rx_time)
        self.assertNotRegex("2:2:8", self.rx_time)
        self.assertRegex("2:32:38", self.rx_time)
        self.assertRegex("9:37:54", self.rx_time)

    def test_metre_regex(self):
        self.assertRegex("0.05_m", self.rx_metre)
        self.assertRegex("15_m", self.rx_metre)
        self.assertRegex("10_m", self.rx_metre)
        self.assertRegex("10.0_m", self.rx_metre)
        self.assertRegex("10.000_m", self.rx_metre)
        self.assertRegex("0.000001_m", self.rx_metre)
        self.assertRegex("5.6_m", self.rx_metre)
        self.assertRegex("5.6_m", self.rx_metre)
        self.assertRegex("51321321331.6846548756735_m", self.rx_metre)
        self.assertRegex("0.0_m", self.rx_metre)
        self.assertRegex("99_m", self.rx_metre)
        self.assertRegex("-1_m", self.rx_metre)
        self.assertRegex("-1.0_m", self.rx_metre)
        self.assertRegex("-11111111.01231212131231232332_m", self.rx_metre)
        self.assertNotRegex("1_km", self.rx_metre)
        self.assertNotRegex("3_m ", self.rx_metre)
        self.assertNotRegex("3.0_m ", self.rx_metre)
        self.assertNotRegex(" 3.0_m", self.rx_metre)
        self.assertNotRegex(" 3_m", self.rx_metre)
        self.assertNotRegex("0.0.1_m", self.rx_metre)
        self.assertRegex("001.1_m", self.rx_metre)
        self.assertRegex("00.1_m", self.rx_metre)
        
    def test_date_format(self):
        with self.assertLogs("main") as context:
            self.checker.checkDateFmt("2002-123-20", 1)
            self.checker.checkDateFmt("2002/12/20", 1)
            self.checker.checkDateFmt("December 12, 2002", 1)
            self.checker.checkDateFmt("20090813", 1)
            self.checker.checkDateFmt("2009-08-33", 1)
            self.checker.checkDateFmt("1-08-03", 1)
            self.checker.checkDateFmt("2011-88-03", 1)
        self.assertEqual(len(context.output), 7)
            
    def test_extra_columns(self):
        with self.assertLogs("main") as context:
            self.checker.checkExtraColNames(["test"])
            self.checker.checkExtraColNames(self.expected + ["0.05a_m"])
            self.checker.checkExtraColNames(self.expected[:1] + ["timezone"] + self.expected[1:])
        self.assertEqual(len(context.output), 3)
        
    def test_col_order(self):
        with self.assertLogs("main") as context:
            self.checker.checkColNames(["a"])
            self.checker.checkColNames(["project_name", "latitude", "longitude", "date_YYYY-MM-DD", "time_HH:MM:SS"])
            self.checker.checkColNames(["site_id", "project_name", "latitude", "longitude", "date_YYYY-MM-DD", "time_HH:MM:SS"])
        # 7 errors because "a"
        # then "latitude", "longitude", "date_YYYY-MM-DD", "time_HH:MM:SS" are all out of the expected order
        # then "site_id" and "project_name" are not in the correct order but the rest of the columns are
        self.assertEqual(len(context.output), 7)

    def test_temperatures(self):
        with self.assertLogs("main") as context:
            self.checker.checkTemperatures("nan", 1)
            self.checker.checkTemperatures("a", 1)
        self.assertEqual(len(context.output), 2)
        
    def test_latitudes(self):
        with self.assertLogs("main") as context:
            self.checker.checkLatitude("abc", 1)
            self.checker.checkLatitude("91", 1)
            self.checker.checkLatitude("-91", 1)
        self.assertEqual(len(context.output), 3)     
        
    def test_longitudes(self):
        with self.assertLogs("main") as context:
            self.checker.checkLatitude("abc", 1)
            self.checker.checkLatitude("181", 1)
            self.checker.checkLatitude("-181", 1)
        self.assertEqual(len(context.output), 3)
        
    def test_unique_timestamps(self):
        dates = pd.Series(["2020-01-01", "2020-01-02", "2020-01-01"])
        times = pd.Series(["00:00:00", "00:02:00", "00:00:00"])
        with self.assertLogs("main") as context:
            self.checker.checkUniqueTimes(dates, times)
        self.assertEqual(len(context.output), 1)

if __name__ == "__main__":
    unittest.main()



    
ERDDAP_BLOCK = XMLchunk = """
        {ERDDAP_HEADER}

        <addAttributes>
            <att name="creator_type">person</att>
            <att name="creator_url">https://webapps.nwtgeoscience.ca/WebAppsV2/Searching/ReferenceSearch.aspx</att>
            <att name="history">{B["Drilling method"]} drilling by {B["Collecting organization"]} (Project #{B["Project number"]}).</att>
            <att name="infoUrl">https://webapps.nwtgeoscience.ca/WebAppsV2/Searching/ReferenceSearch.aspx</att>
            <att name="license">The use of the published data will not carry restrictions. Full citation of referenced publications and reports by users is required.</att>
            <att name="sourceUrl">(local files)</att>
            <att name="standard_name_vocabulary">CF Standard Name Table v55</att>
            <att name="summary">Ground temperature for {borehole_id}.</att>
            <att name="title">Cryostratigraphic data for borehole {borehole_id}</att>
        </addAttributes>
        <dataVariable>
            <sourceName>latitude</sourceName>
            <destinationName>latitude</destinationName>
            <dataType>float</dataType>

            <addAttributes>
                <att name="colorBarMaximum" type="double">90.0</att>
                <att name="colorBarMinimum" type="double">-90.0</att>
                <att name="ioos_category">Location</att>
                <att name="long_name">Latitude</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>longitude</sourceName>
            <destinationName>longitude</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="colorBarMaximum" type="double">180.0</att>
                <att name="colorBarMinimum" type="double">-180.0</att>
                <att name="ioos_category">Location</att>
                <att name="long_name">Longitude</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>time</sourceName>
            <destinationName>time</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="ioos_category">Time</att>
                <att name="long_name">Time</att>
                <att name="units">days since 1990-01-01T00:00:00Z</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>borehole</sourceName>
            <destinationName>borehole</destinationName>
            <dataType>String</dataType>
            <addAttributes>
                <att name="ioos_category">Identifier</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>depth</sourceName>
            <destinationName>depth</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="bounds">null</att>
                <att name="colorBarMaximum" type="double">8000.0</att>
                <att name="colorBarMinimum" type="double">-8000.0</att>
                <att name="colorBarPalette">TopographyDepth</att>
                <att name="ioos_category">Location</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>frozen</sourceName>
            <destinationName>frozen</destinationName>
            <dataType>int</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="coordinates">null</att>
                <att name="ioos_category">Soils</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>cryostructures</sourceName>
            <destinationName>cryostructures</destinationName>
            <dataType>String</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="coordinates">null</att>
                <att name="ioos_category">Unknown</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>visible_ice</sourceName>
            <destinationName>visible_ice</destinationName>
            <dataType>String</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="coordinates">null</att>
                <att name="ioos_category">Ice Distribution</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>ASTM_2488</sourceName>
            <destinationName>ASTM_2488</destinationName>
            <dataType>String</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="coordinates">null</att>
                <att name="ioos_category">Unknown</att>
                <att name="long_name">A description of the interval according to ASTM-2488</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>materials</sourceName>
            <destinationName>materials</destinationName>
            <dataType>String</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="coordinates">null</att>
                <att name="ioos_category">Unknown</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>organic_cover</sourceName>
            <destinationName>organic_cover</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="coordinates">null</att>
                <att name="ioos_category">Unknown</att>
                <att name="long_name">The thickness of the organic cover according to field borehole logs (if present)</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>top_of_interval</sourceName>
            <destinationName>top_of_interval</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="colorBarMaximum" type="double">8000.0</att>
                <att name="colorBarMinimum" type="double">-8000.0</att>
                <att name="colorBarPalette">TopographyDepth</att>
                <att name="ioos_category">Location</att>
            </addAttributes>
        </dataVariable>
        <dataVariable>
            <sourceName>bottom_of_interval</sourceName>
            <destinationName>bottom_of_interval</destinationName>
            <dataType>float</dataType>
            <addAttributes>
                <att name="_ChunkSizes">null</att>
                <att name="colorBarMaximum" type="double">8000.0</att>
                <att name="colorBarMinimum" type="double">-8000.0</att>
                <att name="colorBarPalette">TopographyDepth</att>
                <att name="ioos_category">Location</att>
            </addAttributes>
        </dataVariable>
    </dataset>
    """


ERDDAP_HEADER = """
        <dataset type="EDDTableFromNcCFFiles" datasetID="{dataset_id}" active="true">
        <reloadEveryNMinutes>10080</reloadEveryNMinutes>
        <updateEveryNMillis>10000</updateEveryNMillis>
        <fileDir>{fileDir}</fileDir>
        <fileNameRegex>{Path(fname).name}</fileNameRegex>
        <recursive>true</recursive>
        <pathRegex>.*</pathRegex>
        <metadataFrom>last</metadataFrom>
        <standardizeWhat>0</standardizeWhat>
        <sortFilesBySourceNames></sortFilesBySourceNames>
        <fileTableInMemory>false</fileTableInMemory>
        <accessibleViaFiles>false</accessibleViaFiles>
        """



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='pfit',
    version='0.2.0',
    description='Tools to support interoperability and the adoption of standards for permafrost data files.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/permafrostnet/pfit',
    author='PermafrostNet',
    author_email='nick.brown@carleton.ca',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)', 
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],
    keywords='ground, temperature, sensor, permafrost, earth, science, soil, data',
    packages=find_packages(),
    python_requires='>=3.6, <4',
    install_requires=['numpy', 'pandas', 'netcdf4', 'xlrd'],
    project_urls={
        'Bug Reports': 'https://gitlab.com/permafrostnet/pfit/-/issues',
        'Source': 'https://gitlab.com/permafrostnet/pfit',
    },
)


""" CF-style metdata key-value pairs
"""
ntgs_global_default = {
    "publisher_name": "Northwest Territories Geological Survey",
    "publisher_email": "NTGS@gov.nt.ca",
    "publisher_url": "https://www.nwtgeoscience.ca/",
    "publisher_type": "institution",
    "license": "The use of the published data will not carry restrictions. Full citation of referenced publications and reports by users is required.",
    "publisher_institution": "Northwest Territories Geological Survey",
}


"""
These map NTGS excel sheets into ACDD
# "ACDD attribute name" : ("NTGS Excel sheet", "Column Name")
"""

ACDD_globals = {
    "institution": ("Project Details", "Organization (mandatory)"),
    "creator_email": ("Project Details", "Email (mandatory)"),
    "geospatial_vertical_min": ("Location", "Site Elevation (m) (mandatory)"),
    "geospatial_vertical_max": ("Location", "Site Elevation (m) (mandatory)"),
    "geospatial_lat_min": ("Location", "Latitude (mandatory)"),
    "geospatial_lat_max": ("Location", "Latitude (mandatory)"),
    "geospatial_lon_min": ("Location", "Longitude (mandatory)"),
    "geospatial_lon_max": ("Location", "Longitude (mandatory)"),
    "project": ("Project Details", "Project Name")
}

ACDD_variable_attributes = {
    "ground_temperature": {
            "resolution": ("GTC Installation", "Sensor Resolution (˚C)"),
            "accuracy": ("GTC Installation", "Sensor Accuracy (˚C)")
          }

}

Permafrost_globals = {
    "ground_slope_angle": ("Site Conditions", "Slope Angle"),
    "ground_slope_direction": ("Site Conditions", "Slope Aspect"),
    "vegetation_type": ("Site Conditions", "Dominant Vegetation (mandatory)"),
    "organic_matter_thickness": ("Site Conditions", "Organic Layer"),
    "overburden_thickness": ("Site Conditions", "Overburden Thickness (m)"),
    "surface_cover": ("Site Conditions", "Surface Cover Material"),
    "surficial_geology": ("Site Conditions", "Surficial Geology (mandatory)"),
    "environment_description" : ("Site Conditions", "Site Conditions-Additional Comments")
}

IOOS_globals = {

}

v_names = {
    'temperature': 'ground_temperature',
    'elevation': 'elevation'
}




import pandas as pd
import numpy as np
import re
import netCDF4 as nc
#from fuzzywuzzy import fuzz

from pfit.MetadataHandler import MetadataHandler
from pfit.ntgs_metdata import ACDD_globals, ACDD_variable_attributes, ntgs_global_default, v_names, Permafrost_globals

class NTGSTemperatureMetadataHandler(MetadataHandler):

    def __init__(self, xl_file):
        """ Reads and validates NTGS temperature metadata file """
        self.dir = {}
        self.file = xl_file
        self.readxl(xl_file)
        self._selected_site = None
    
    def __getitem__(self, item):
        return self.dir.get(item, "")

    @classmethod
    def from_xlsx(cls, xl_file):
        """ same as the default constructor for now"""
        C = cls(xl_file)
        return C

    def readxl(self, xl_file):
        self.read_tab(xl_file, "Location")
        self.read_tab(xl_file, "GTC Installation")
        self.read_tab(xl_file, "Ground Temperature Record")
        self.read_tab(xl_file, "Project Details")
        self.read_tab(xl_file, "Site Conditions")

    @property    
    def selected_site(self):
        if self._selected_site is None: 
            self.selected_site = 1
            print("No site selected. Selecting first record")    
        
        return self._selected_site
        
    @selected_site.setter
    def selected_site(self, value):
        """ Set the selected location
        site_id : int or str
            row number or site name 
            """
        if isinstance(value, int):
            self._selected_site = self["Project Details"].loc[value, "Site ID (mandatory)"]
        
        elif isinstance(value, str):
            if not value in self.get_sites():
                raise KeyError("Chosen site is not in the Project Details tab. Selection not possible")
            self._selected_site = value

    def get_sites(self, tab="Project Details"):
        if tab == "Project Details":
            return self["Project Details"]["Site ID (mandatory)"].values
        else:
            return self["Location"]["Site ID"].values

    def read_tab(self, xl_file, name):
        tab = pd.read_excel(xl_file, skiprows=[1], sheet_name=name, engine='openpyxl')
        self.dir[name] = tab.loc[~tab.iloc[:, 0].isna()]  # 0-index because name for Site ID changes by tab
        self._validate_tab(name)

    def read_disclaimer(self, xl_file):
        text = pd.read_excel(xl_file, sheet_name="Disclaimer-Copyright", engine='openpyxl').to_json()
        citation = re.search(r'Recommended citation:\s*(.*)"', text).group(1).replace(r"\n", "")
        terms = re.search('Terms of use(.*?)"', text).group(1).replace(r"\n", "")

    def _validate_tab(self, tab):
        validator = self._get_validator(tab)
        validator(self[tab])

    def _get_validator(self, tab):
        if tab == "Location":
            return self._validate_location_tab
        # elif tab == "GTC Installation":
        #     return self.validate_gtc_tab
        # elif tab == "Site Conditions":
        #     return self.validate_site_tab
        # elif tab == "Project Details":
        #     return self.validate_project_tab
        else:
            return self.null_validator

    def null_validator(self, tab):
        pass

    def _validate_location_tab(self, df):
        self._validate_location_headers(df)
        self.validate_latitude(df["Latitude (mandatory)"].to_numpy())

    def _validate_location_headers(self, df):
        for header in ["Site ID", "Latitude (mandatory)", "Longitude (mandatory)",
                        "Geodetic Datum", "Site Elevation (m) (mandatory)"]:
            if not header in df.columns:
                print(f"Warning, header {header} missing.")

    def match_name(self, tab, name):
        clean_name = name.strip().lower()
        metadata_sites = {s.strip().lower(): s for s in self.get_sites(tab)}
        
        if not clean_name in metadata_sites.keys():
            raise KeyError(f"Site name {name} not in {self.get_sites(tab)} on tab {tab}")
        else:
            return metadata_sites[clean_name]

    def get_value(self, tab, col, loc):
        site_name = self.match_name(tab, loc)
        return self[tab][col][np.where(self.get_sites(tab) == site_name)[0][0]]

    def get_ACDD_attr(self, attr, loc=None):
        return self.get_translated_attr(attr, ACDD_globals, loc=None)

    def get_translated_attr(self, attr, dictionary, loc=None):
        if loc is None:
            loc = self.selected_site

        tab, col = dictionary[attr]
        
        return self.get_value(tab, col, loc)
    @staticmethod
    def validate_latitude(lats):
        """[summary]

        Parameters
        ----------
        lats : array-like
            latitude values
        """
        assert(max(lats) >= -90)
        assert(min(lats) <= 90 )

    def write_attributes(self, nc_file, loc):
        with nc.Dataset(nc_file, 'a') as rootgrp:
            for att in ACDD_globals: 
                value = self.get_ACDD_attr(att, loc)
                rootgrp.setncattr(att, value)
            for att in Permafrost_globals:
                value = self.get_translated_attr(att, Permafrost_globals, loc)
                rootgrp.setncattr(att, value)
            for att, value in ntgs_global_default.items():
                rootgrp.setncattr(att, value)
    
    def write_variables(self, nc_file, loc):
        self._add_elevation(nc_file, loc)

    def _add_elevation(self, nc_file, loc):
        with nc.Dataset(nc_file, 'a') as rootgrp:
            rootgrp[v_names['elevation']][:] = self.get_value("Location","Site Elevation (m) (mandatory)", loc)



if __name__ == "__main__":
    
    xl_file = r"C:\Users\Nick\Downloads\NTGSref_2020April23\2019-007_2019-007\2019-007_Metadata\NWT_Open_Report_2019-007_metadata.xlsx"
    M = NTGSTemperatureMetadataHandler(xl_file)
    M.get_ACDD_attr('institution')